==> libremail-2.3.tar.bz2: libremail/src/joindre.c <==
/*
    Fichier joindre.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence-fr.txt

    Traductions des droits d'utilisation dans les fichiers :
    licence-de.txt , licence-en.txt , licence-es.txt ,
    licence-it.txt , licence-nl.txt , licence-pt.txt ,
    licence-eo.txt , licence-eo-utf.txt

    Droits d'utilisation galement sur la page web :
    http://libremail.tuxfamily.org/voir.php?page=droits


    Ce programme rajoute des fichiers joints  un mail  envoyer.
    Le nom du fichier mail concern est pass en paramtre.
    Le chemin d'accs au fichier peut tre absolu ou relatif
*/

#define appli   // pour la dclaration de variables globales  l'application

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "messages.h"
#include "buflect.h"
#include "fmail.h"
#include "trtentete.h"
#include "trtligne.h"
#include "trtbordure.h"
#include "base64.h"


#define  octet     unsigned char


/* prototypes */
void prepar_mail ();
void ajout_fich (int narg, char **varg);
void joinfich (char *chemfich);
void ajout_quoted ();
void ajout_base64 ();


/* variables globales au source
   (pour viter des tonnes de passages de paramtres) */

FILE *descfic;       // descripteur du fichier  joindre
int  ajouts = 0;     // pour viter dfauts si aucun fichier n'est rajout



/* programme principal */

int main (int nbarg, char *varg[])
{
    // rcupration du nom de l'excutable
    memcom (*varg);

    // controle du nombre d'arguments
    if (--nbarg >= 1)
    {
        // si le fichier mail existe
        if (access (varg [1], 0) == 0)
        {
            // passer le mail en mode multipart/mixed si ncessaire
            prepar_mail (varg [1]);

            // ajouter les fichiers  joindre
            ajout_fich (--nbarg, varg + 1);
        }
        else
            // "Fichier %s non trouv"
            aff_err_arg ("FICH_ABSENT", varg [1]);
    }
    else
        // "Syntaxe : %s nom_fichier_mail [fichiers__joindre]"
        psyntaxe ("SYNT_JOINDRE");

    // pour faire plaisir  gcc qui veut une fonction main de type int
    return (0);
}



/* passe le mail en mode multipart/mixed s'il n'y est pas dj */

void prepar_mail (char *nomfmail)
{
    long posfic;         // position dans le fichier en dbut de ligne
    long posContent;     // position au dbut du champ Content-Type
    char bufEncode [80]; // contenu du champ Content-Transfer-Encoding
    char fictmp [120];   // fichier tampon pour passer en mode multipart/mixed
    FILE *desctmp;       // descripteur du fichier fictmp
    int  typederlig;     // mmorise si la dernire ligne du message est blanche


    // initialisation
    posfic     =  0;
    posContent = -1;
    *bufEncode = '\0';

    // ouvrir le fichier mail en lecture/criture
    fmail = fopen (nomfmail, "r+");

    // erreur fatale si ouverture refuse
    if (! fmail)
    {
        // "Impossible de mettre  jour le fichier %s"
        errfatale ("IMPOS_MAJ_FICH", nomfmail);
    }

    // lecture du fichier mail jusqu'au champ Content-Type
    while (lire_fmail () && ! start ("Content-Type"))
        ;

    // si le fichier n'est pas encore mode multipart/mixed
    if (recup_ctype () != MultipMixed)
    {
        // fabriquer un nom de fichier tampon
        strcpy (fictmp, nomfmail);
        strcat (fictmp, ".t");

        // et l'ouvrir en criture
        desctmp = fopen (fictmp, "w");

        // erreur fatale si ouverture refuse
        if (! desctmp)
        {
            // "Impossible de mettre  jour le fichier %s"
            errfatale ("IMPOS_MAJ_FICH", nomfmail);
        }

        // revenir au debut du fichier mail
        rewind (fmail);
        lire_fmail ();

        // recopier son entte sans les champs Content-T...
        while (lire_fmail () && *buf_lect)
        {
            // si champ Content-Type
            if (start ("Content-Type"))
            {
                // on mmorise la position de la ligne
                posContent = posfic;

                // et on lit jusqu' la dernire ligne du champ
                do
                    lire_fmail ();
                while (*buf_lect == ' ' || *buf_lect == '\t');
            }

            // si champ Content-Transfer-Encoding
            if (start ("Content-Transfer-Encoding"))
                // on mmorise la ligne
                membuf (bufEncode);
            // sinon
            else
            {
                // on recopie le ligne dans le fichier tampon
                fputs (buf_lect, desctmp);
                fputc ('\n', desctmp);
            }

            // rcuprer la position de dbut de la ligne suivante
            posfic = ftell (fmail);
        }

        // gnrer une bordure avec une partie alatoire
        sprintf (bordure [0], "-LIBREMAIL-BORDURE-id=%08X%03X-", time (0),
                                                                 getpid ());

        // gnrer le nouveau champ Content-Type
        fprintf (desctmp, "Content-Type: multipart/mixed; boundary=\"%s\"\n",
                                                                 bordure [0]);

        // premire section : gnrer la bordure
        fprintf (desctmp, "\n--%s\n", bordure [0]);

        // le champ Content-Type
        if (posContent >= 0)
        {
            // on se positionne dessus pour le recopier
            fseek (fmail, posContent, SEEK_SET);
            lire_fmail ();

            // rpter
            do
            {
                // recopier la ligne
                fprintf (desctmp, "%s\n", buf_lect);

                // et lire la suivante
                lire_fmail ();
            }
            // jusqu' la fin de ce champ
            while (*buf_lect == ' ' || *buf_lect == '\t');
        }
        else
        {
            // jeu de caractres du message
            fputs ("Content-Type: text/plain; charset=", desctmp);

            if (util_utf8 ())
                fputs ("UTF-8;\n", desctmp);
            else
                fputs ("ISO-8859-15;\n", desctmp);
        }

        // le champ Content-Transfer-Encoding
        if (*bufEncode)
            fputs (bufEncode, desctmp);
        else
            fputs ("Content-Transfer-Encoding: 8bit", desctmp);

        // + passage  la ligne
        fputc ('\n', desctmp);

        // revenir sur le texte du mail
        fseek (fmail, posfic, SEEK_SET);

        // le recopier jusqu'en fin de fichier
        while (lire_fmail ())
        {
            // recopier la ligne
            fprintf (desctmp, "%s\n", buf_lect);

            // mmoriser si longueur nulle
            typederlig = *buf_lect;
        }

        // rajouter une ligne blanche si ncessaire
        if (typederlig)
            fputc ('\n', desctmp);

        // gnrer la bordure finale
        fprintf (desctmp, "--%s--\n", bordure [0]);

        // fermer les fichiers
        fclose (desctmp);
        fclose (fmail);

        // et remplacer l'ancien fichier par le nouveau
        unlink (nomfmail);
        rename (fictmp, nomfmail);
    }
    // sinon (fichier dj en mode multipart/mixed)
    else
    {
        // mmoriser la bordure
        while (*buf_lect && ! mem_boundary ())
            lire_fmail ();

        // et fermer le fichier (on le rouvrira dans ajout_fich)
        fclose (fmail);
    }
}



/* rajoute les fichiers joints au mail */

void ajout_fich (int narg, char **varg)
{
    char chemfich [120];   // vom de fichier  joindre saisi au clavier
    int  i;                // compteur


    // ouvrir le fichier mail en lecture/criture
    fmail = fopen (*varg, "r+");

    // traitement d'une erreur qui ne devrait jamais arriver
    if (!fmail)
    {
        // "Erreur 2me ouverture fichier mail !!!"
        affiche_err ("ERR_OUVERT_2");
        return;
    }

    // si des noms de fichiers  joindre ont t passs en paramtre
    if (narg > 0)
    {
        // les rajouter en pice jointe un par un
        for (i = 1; i <= narg; i++)
            joinfich (varg [i]);
    }
    // sinon, on va saisir leur nom au clavier
    else
    {
        // retour  la configuration standard du clavier si ncessaire
        system ("stty icanon echo");

        do
        {
         // "Nom de fichier  joindre ? (touche entre quand il n'y en a plus)"
            affiche_msg ("NOUV_FICJOINT");

            // on lit avec fgets pour viter le warning de compilation de gets
            // puis on enlve le \n en trop  (que gets ne met pas)
            fgets (chemfich, sizeof (chemfich), stdin);
            chemfich [strlen (chemfich) - 1] = '\0';

            if (*chemfich)
                joinfich (chemfich);
        }
        while (*chemfich);   // liste termine lorsque ligne vide
    }

    // si on a rajout des fichiers
    if (ajouts)
    {
        // corriger la dernire bordure
        fseek (fmail, -1, SEEK_END);
        fputs ("--\n", fmail);
    }

    // le fichier mail est prt
    fclose (fmail);
}



/* rajoute un fichier joint au mail */

void joinfich (char *chemfich)
{
    int  car;              // caractre du fichier  joindre
    int  normal, special;  // compteurs de caractres
    char nomfich [30];     // nom du fichier joint (sans le chemin)
    int  i, j;             // compteurs


    // ouvrir en lecture le fichier  joindre
    descfic = fopen (chemfich, "r");

    // si ouvert
    if (descfic)
    {
        // initialisation
        normal  = 0;
        special = 0;

        // on va dterminer le mode d'encodage le moins encombrant
        car = getc (descfic);

        // tant que non fin de fichier
        while (car != EOF)
        {
            // comptabiliser ce caractre dans la bonne catgorie
            if ((car < 0x20 && car != '\t') || car == '=' || car > 0X7E)
                special++;
            else
                normal++;

            // continuer avec le caractre suivant
            car = getc (descfic);
        }

        // si pas encore de fichier rajout
        if (!ajouts)
        {
            // supprimer 2 -  la dernire bordure
            fseek (fmail, -3, SEEK_END);
            fputc ('\n', fmail);

            // cette correction n'est faite qu'une fois
            ajouts = 1;
        }

        // fabriquer le nom du fichier joint  partir du chemin
        j = 0;

        for (i = 0; chemfich [i]; i++)
        {
            // on supprime le nom de rpertoire du fichier  joindre
            if (chemfich [i] == '/' || chemfich [i] == '\\')
                j = 0;
            // ainsi que les blancs dans le nom de fichier
            else if (chemfich [i] != ' ')
                nomfich [j++] = chemfich [i];

        }

        nomfich [j] = 0;

        // gnrer l'entte de la pice jointe
        fputs   ("Content-Type: application/octet-stream;\n", fmail);
        fprintf (fmail, "        name=\"%s\"\n", nomfich);
        fputs   ("Content-Disposition: attachment;\n", fmail);
        fprintf (fmail, "        filename=\"%s\"\n", nomfich);

        // revenir en dbut de fichier
        rewind (descfic);

        // choisir le mode d'encodage pour joindre la pice
        if (special * 5 <= normal)
            ajout_quoted ();
        else
            ajout_base64 ();

        // gnrer la bordure terminale
        fprintf (fmail, "--%s\n", bordure [0]);

        // termin avec ce fichier
        fclose (descfic);
    }
    // sinon, message d'erreur
    else if (access (chemfich, 0) < 0)
        // "Fichier %s inexistant"
        aff_err_arg ("FICH_INEXISTANT", chemfich);
    else
        // "Fichier %s protg en lecture"
        aff_err_arg ("ACCES_FICH_LECT", chemfich);
}



/* encode le fichier  joindre en quoted-printable */

void ajout_quoted ()
{
    int car;       // caractres du fichier  joindre
    int posligne;  // position dans la ligne courante du fichier mail


    // terminaison de l'entte de la pice jointe
    fputs ("Content-Transfer-Encoding: quoted-printable\n\n", fmail);

    // initialisation
    posligne = 1;
    car = getc (descfic);

    // tant que non fin de fichier
    while (car != EOF)
    {
        // traitement particulier du caractre CR
        if (car == '\r')
        {
            // lire le caractre suivant
            car = getc (descfic);

            // si c'est un LF
            if (car == '\n')
            {
                // passage  la ligne sans le \r qui
                // sera rajout  l'expdition du mail
                fputc ('\n', fmail);
                posligne = 1;
            }
            // sinon
            else
            {
                // encoder le CR comme les autres caractres spciaux
                fputs ("=0D", fmail);
                posligne = posligne + 3;

                // on traitera le caractre qui suit plus tard
                ungetc (car, descfic);
            }

            // si CR LF un passage  la ligne sera gnr plus loin
        }

        // autres caractres spciaux
        else if ((car < 0x20 && car != '\t') || car == '=' || car > 0X7E)
        {
            // passage  la ligne si la prcdente tait trop longue
            if (posligne > 72)
            {
                fputs ("=\n", fmail);
                posligne = 1;
            }

            // ces caractres sont encods
            fprintf (fmail, "=%02X", car);

            if (car == '\n')
            {
                fputs ("=\n", fmail);   // pour la lisibilit
                posligne = 1;
            }
            else
                posligne = posligne + 3;
        }
        // caractres normaux
        else
        {
            // passage  la ligne si la prcdente tait trop longue
            if (posligne > 74)
            {
                fputs ("=\n", fmail);
                posligne = 1;
            }

            // ces caractres sont juste recopis
            putc (car, fmail);
            posligne ++;
        }

        // passer au caractre suivant
        car = getc (descfic);
    }
}



/* encode le fichier  joindre en base64 */

void ajout_base64 ()
{
    char bufin  [58]; // tableau contenant des donnes du fichier  joindre
    char bufout [77]; // donnes de bufin converties base64
    int  nbcar;       // nombre de caractres rcuprs dans bufin


    // terminaison de l'entte de la pice jointe
    fputs ("Content-Transfer-Encoding: base64\n\n", fmail);

    // lire de quoi gnrer une ligne encode base64
    nbcar = fread (bufin, 1, 57, descfic);

    // si le fichier joint n'est pas vide
    if (nbcar)
    {
        // rpter
        do
        {
            // encoder les donnes base64 et les copier dans le fichier mail
            encode64 (bufin, bufout, nbcar);
            fputs (bufout, fmail);
            fputc ('\n', fmail);

            // lire de quoi gnrer une ligne de plus
            nbcar = fread (bufin, 1, 57, descfic);
        }
        // jusqu' fin de fichier
        while (nbcar);
    }
}

==> libremail-2.5.3.tar.bz2: libremail/src/joindre.c <==
/*
    Fichier joindre.c
    Auteur Bernard Chardonneau

    Logiciel libre, droits d'utilisation prciss en franais
    dans le fichier : licence-fr.txt

    Traductions des droits d'utilisation dans les fichiers :
    licence-de.txt , licence-en.txt , licence-es.txt ,
    licence-it.txt , licence-nl.txt , licence-pt.txt ,
    licence-eo.txt , licence-eo-utf.txt

    Droits d'utilisation galement sur la page web :
    http://libremail.tuxfamily.org/voir.php?page=droits


    Ce programme rajoute des fichiers joints  un mail  envoyer.
    Le nom du fichier mail concern est pass en paramtre.
    Le chemin d'accs au fichier peut tre absolu ou relatif
*/

#define appli   // pour la dclaration de variables globales  l'application

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include "messages.h"
#include "buflect.h"
#include "fmail.h"
#include "trtentete.h"
#include "trtligne.h"
#include "trtbordure.h"
#include "base64.h"


#define  octet     unsigned char


/* prototypes */
void prepar_mail ();
void ajout_fich (int narg, char **varg);
void joinfich (char *chemfich);
void ajout_quoted ();
void ajout_base64 ();


/* variables globales au source
   (pour viter des tonnes de passages de paramtres) */

FILE *descfic;       // descripteur du fichier  joindre
int  ajouts = 0;     // pour viter dfauts si aucun fichier n'est rajout



/* programme principal */

int main (int nbarg, char *varg[])
{
    // rcupration du nom de l'excutable
    memcom (*varg);

    // controle du nombre d'arguments
    if (--nbarg >= 1)
    {
        // si le fichier mail existe
        if (access (varg [1], 0) == 0)
        {
            // passer le mail en mode multipart/mixed si ncessaire
            prepar_mail (varg [1]);

            // ajouter les fichiers  joindre
            ajout_fich (--nbarg, varg + 1);
        }
        else
            // "Fichier %s non trouv"
            aff_err_arg ("FICH_ABSENT", varg [1]);
    }
    else
        // "Syntaxe : %s nom_fichier_mail [fichiers__joindre]"
        psyntaxe ("SYNT_JOINDRE");

    // pour faire plaisir  gcc qui veut une fonction main de type int
    return (0);
}



/* passe le mail en mode multipart/mixed s'il n'y est pas dj */

void prepar_mail (char *nomfmail)
{
    long posfic;         // position dans le fichier en dbut de ligne
    long posContent;     // position au dbut du champ Content-Type
    char bufEncode [80]; // contenu du champ Content-Transfer-Encoding
    char fictmp [120];   // fichier tampon pour passer en mode multipart/mixed
    FILE *desctmp;       // descripteur du fichier fictmp
    int  typederlig;     // mmorise si la dernire ligne du message est blanche


    // initialisation
    posfic     =  0;
    posContent = -1;
    *bufEncode = '\0';

    // ouvrir le fichier mail en lecture/criture
    fmail = fopen (nomfmail, "r+");

    // erreur fatale si ouverture refuse
    if (! fmail)
    {
        // "Impossible de mettre  jour le fichier %s"
        errfatale ("IMPOS_MAJ_FICH", nomfmail);
    }

    // lecture du fichier mail jusqu'au champ Content-Type
    while (lire_fmail () && ! start ("Content-Type"))
        ;

    // si le fichier n'est pas encore mode multipart/mixed
    if (recup_ctype () != MultipMixed)
    {
        // fabriquer un nom de fichier tampon
        strcpy (fictmp, nomfmail);
        strcat (fictmp, ".t");

        // et l'ouvrir en criture
        desctmp = fopen (fictmp, "w");

        // erreur fatale si ouverture refuse
        if (! desctmp)
        {
            // "Impossible de mettre  jour le fichier %s"
            errfatale ("IMPOS_MAJ_FICH", nomfmail);
        }

        // revenir au debut du fichier mail
        rewind (fmail);
        lire_fmail ();

        // recopier son entte sans les champs Content-T...
        while (lire_fmail () && *buf_lect)
        {
            // si champ Content-Type
            if (start ("Content-Type"))
            {
                // on mmorise la position de la ligne
                posContent = posfic;

                // et on lit jusqu' la dernire ligne du champ
                do
                    lire_fmail ();
                while (*buf_lect == ' ' || *buf_lect == '\t');
            }

            // si champ Content-Transfer-Encoding
            if (start ("Content-Transfer-Encoding"))
                // on mmorise la ligne
                membuf (bufEncode);
            // sinon
            else
            {
                // on recopie le ligne dans le fichier tampon
                fputs (buf_lect, desctmp);
                fputc ('\n', desctmp);
            }

            // rcuprer la position de dbut de la ligne suivante
            posfic = ftell (fmail);
        }

        // gnrer une bordure avec une partie alatoire
        sprintf (bordure [0], "-LIBREMAIL-BORDURE-id=%08lX%03X-", time (0),
                                                                 getpid ());

        // gnrer le nouveau champ Content-Type
        fprintf (desctmp, "Content-Type: multipart/mixed; boundary=\"%s\"\n",
                                                                 bordure [0]);

        // premire section : gnrer la bordure
        fprintf (desctmp, "\n--%s\n", bordure [0]);

        // le champ Content-Type
        if (posContent >= 0)
        {
            // on se positionne dessus pour le recopier
            fseek (fmail, posContent, SEEK_SET);
            lire_fmail ();

            // rpter
            do
            {
                // recopier la ligne
                fprintf (desctmp, "%s\n", buf_lect);

                // et lire la suivante
                lire_fmail ();
            }
            // jusqu' la fin de ce champ
            while (*buf_lect == ' ' || *buf_lect == '\t');
        }
        else
        {
            // jeu de caractres du message
            fputs ("Content-Type: text/plain; charset=", desctmp);

            if (util_utf8 ())
                fputs ("UTF-8;\n", desctmp);
            else
                fputs ("ISO-8859-15;\n", desctmp);
        }

        // le champ Content-Transfer-Encoding
        if (*bufEncode)
            fputs (bufEncode, desctmp);
        else
            fputs ("Content-Transfer-Encoding: 8bit", desctmp);

        // + passage  la ligne
        fputc ('\n', desctmp);

        // revenir sur le texte du mail
        fseek (fmail, posfic, SEEK_SET);

        // le recopier jusqu'en fin de fichier
        while (lire_fmail ())
        {
            // recopier la ligne
            fprintf (desctmp, "%s\n", buf_lect);

            // mmoriser si longueur nulle
            typederlig = *buf_lect;
        }

        // rajouter une ligne blanche si ncessaire
        if (typederlig)
            fputc ('\n', desctmp);

        // gnrer la bordure finale
        fprintf (desctmp, "--%s--\n", bordure [0]);

        // fermer les fichiers
        fclose (desctmp);
        fclose (fmail);

        // et remplacer l'ancien fichier par le nouveau
        unlink (nomfmail);
        rename (fictmp, nomfmail);
    }
    // sinon (fichier dj en mode multipart/mixed)
    else
    {
        // mmoriser la bordure
        while (*buf_lect && ! mem_boundary ())
            lire_fmail ();

        // et fermer le fichier (on le rouvrira dans ajout_fich)
        fclose (fmail);
    }
}



/* rajoute les fichiers joints au mail */

void ajout_fich (int narg, char **varg)
{
    char chemfich [120];   // vom de fichier  joindre saisi au clavier
    int  i;                // compteur


    // ouvrir le fichier mail en lecture/criture
    fmail = fopen (*varg, "r+");

    // traitement d'une erreur qui ne devrait jamais arriver
    if (!fmail)
    {
        // "Erreur 2me ouverture fichier mail !!!"
        affiche_err ("ERR_OUVERT_2");
        return;
    }

    // si des noms de fichiers  joindre ont t passs en paramtre
    if (narg > 0)
    {
        // les rajouter en pice jointe un par un
        for (i = 1; i <= narg; i++)
            joinfich (varg [i]);
    }
    // sinon, on va saisir leur nom au clavier
    else
    {
        // retour  la configuration standard du clavier si ncessaire
        system ("stty icanon echo");

        do
        {
         // "Nom de fichier  joindre ? (touche entre quand il n'y en a plus)"
            affiche_msg ("NOUV_FICJOINT");

            // on lit avec fgets pour viter le warning de compilation de gets
            // puis on enlve le \n en trop  (que gets ne met pas)
            fgets (chemfich, sizeof (chemfich), stdin);
            chemfich [strlen (chemfich) - 1] = '\0';

            if (*chemfich)
                joinfich (chemfich);
        }
        while (*chemfich);   // liste termine lorsque ligne vide
    }

    // si on a rajout des fichiers
    if (ajouts)
    {
        // corriger la dernire bordure
        fseek (fmail, -1, SEEK_END);
        fputs ("--\n", fmail);
    }

    // le fichier mail est prt
    fclose (fmail);
}



/* rajoute un fichier joint au mail */

void joinfich (char *chemfich)
{
    int  car;              // caractre du fichier  joindre
    int  normal, special;  // compteurs de caractres
    char nomfich [30];     // nom du fichier joint (sans le chemin)
    int  i, j;             // compteurs


    // ouvrir en lecture le fichier  joindre
    descfic = fopen (chemfich, "r");

    // si ouvert
    if (descfic)
    {
        // initialisation
        normal  = 0;
        special = 0;

        // on va dterminer le mode d'encodage le moins encombrant
        car = getc (descfic);

        // tant que non fin de fichier
        while (car != EOF)
        {
            // comptabiliser ce caractre dans la bonne catgorie
            if ((car < 0x20 && car != '\t') || car == '=' || car > 0X7E)
                special++;
            else
                normal++;

            // continuer avec le caractre suivant
            car = getc (descfic);
        }

        // si pas encore de fichier rajout
        if (!ajouts)
        {
            // supprimer 2 -  la dernire bordure
            fseek (fmail, -3, SEEK_END);
            fputc ('\n', fmail);

            // cette correction n'est faite qu'une fois
            ajouts = 1;
        }

        // fabriquer le nom du fichier joint  partir du chemin
        j = 0;

        for (i = 0; chemfich [i]; i++)
        {
            // on supprime le nom de rpertoire du fichier  joindre
            if (chemfich [i] == '/' || chemfich [i] == '\\')
                j = 0;
            // ainsi que les blancs dans le nom de fichier
            else if (chemfich [i] != ' ')
                nomfich [j++] = chemfich [i];

        }

        nomfich [j] = 0;

        // gnrer l'entte de la pice jointe
        fputs   ("Content-Type: application/octet-stream;\n", fmail);
        fprintf (fmail, "        name=\"%s\"\n", nomfich);
        fputs   ("Content-Disposition: attachment;\n", fmail);
        fprintf (fmail, "        filename=\"%s\"\n", nomfich);

        // revenir en dbut de fichier
        rewind (descfic);

        // choisir le mode d'encodage pour joindre la pice
        if (special * 5 <= normal)
            ajout_quoted ();
        else
            ajout_base64 ();

        // gnrer la bordure terminale
        fprintf (fmail, "--%s\n", bordure [0]);

        // termin avec ce fichier
        fclose (descfic);
    }
    // sinon, message d'erreur
    else if (access (chemfich, 0) < 0)
        // "Fichier %s inexistant"
        aff_err_arg ("FICH_INEXISTANT", chemfich);
    else
        // "Fichier %s protg en lecture"
        aff_err_arg ("ACCES_FICH_LECT", chemfich);
}



/* encode le fichier  joindre en quoted-printable */

void ajout_quoted ()
{
    int car;       // caractres du fichier  joindre
    int posligne;  // position dans la ligne courante du fichier mail


    // terminaison de l'entte de la pice jointe
    fputs ("Content-Transfer-Encoding: quoted-printable\n\n", fmail);

    // initialisation
    posligne = 1;
    car = getc (descfic);

    // tant que non fin de fichier
    while (car != EOF)
    {
        // traitement particulier du caractre CR
        if (car == '\r')
        {
            // lire le caractre suivant
            car = getc (descfic);

            // si c'est un LF
            if (car == '\n')
            {
                // passage  la ligne sans le \r qui
                // sera rajout  l'expdition du mail
                fputc ('\n', fmail);
                posligne = 1;
            }
            // sinon
            else
            {
                // encoder le CR comme les autres caractres spciaux
                fputs ("=0D", fmail);
                posligne = posligne + 3;

                // on traitera le caractre qui suit plus tard
                ungetc (car, descfic);
            }

            // si CR LF un passage  la ligne sera gnr plus loin
        }

        // autres caractres spciaux
        else if ((car < 0x20 && car != '\t') || car == '=' || car > 0X7E)
        {
            // passage  la ligne si la prcdente tait trop longue
            if (posligne > 72)
            {
                fputs ("=\n", fmail);
                posligne = 1;
            }

            // ces caractres sont encods
            fprintf (fmail, "=%02X", car);

            if (car == '\n')
            {
                fputs ("=\n", fmail);   // pour la lisibilit
                posligne = 1;
            }
            else
                posligne = posligne + 3;
        }
        // caractres normaux
        else
        {
            // passage  la ligne si la prcdente tait trop longue
            if (posligne > 74)
            {
                fputs ("=\n", fmail);
                posligne = 1;
            }

            // ces caractres sont juste recopis
            putc (car, fmail);
            posligne ++;
        }

        // passer au caractre suivant
        car = getc (descfic);
    }
}



/* encode le fichier  joindre en base64 */

void ajout_base64 ()
{
    char bufin  [58]; // tableau contenant des donnes du fichier  joindre
    char bufout [77]; // donnes de bufin converties base64
    int  nbcar;       // nombre de caractres rcuprs dans bufin


    // terminaison de l'entte de la pice jointe
    fputs ("Content-Transfer-Encoding: base64\n\n", fmail);

    // lire de quoi gnrer une ligne encode base64
    nbcar = fread (bufin, 1, 57, descfic);

    // si le fichier joint n'est pas vide
    if (nbcar)
    {
        // rpter
        do
        {
            // encoder les donnes base64 et les copier dans le fichier mail
            encode64 (bufin, bufout, nbcar);
            fputs (bufout, fmail);
            fputc ('\n', fmail);

            // lire de quoi gnrer une ligne de plus
            nbcar = fread (bufin, 1, 57, descfic);
        }
        // jusqu' fin de fichier
        while (nbcar);
    }
}
